@@ -1,6 +1,7 @@
 """Analyze integration test target code coverage."""
 from __future__ import annotations
 
+import collections.abc as c
 import os
 import typing as t
 
@@ -18,22 +19,23 @@ from .. import (
     CoverageAnalyzeConfig,
 )
 
-TargetKey = t.TypeVar('TargetKey', int, t.Tuple[int, int])
-NamedPoints = t.Dict[str, t.Dict[TargetKey, t.Set[str]]]
-IndexedPoints = t.Dict[str, t.Dict[TargetKey, t.Set[int]]]
-Arcs = t.Dict[str, t.Dict[t.Tuple[int, int], t.Set[int]]]
-Lines = t.Dict[str, t.Dict[int, t.Set[int]]]
-TargetIndexes = t.Dict[str, int]
-TargetSetIndexes = t.Dict[t.FrozenSet[int], int]
+TargetKey = t.TypeVar('TargetKey', int, tuple[int, int])
+TFlexKey = t.TypeVar('TFlexKey', int, tuple[int, int], str)
+NamedPoints = dict[str, dict[TargetKey, set[str]]]
+IndexedPoints = dict[str, dict[TargetKey, set[int]]]
+Arcs = dict[str, dict[tuple[int, int], set[int]]]
+Lines = dict[str, dict[int, set[int]]]
+TargetIndexes = dict[str, int]
+TargetSetIndexes = dict[frozenset[int], int]
 
 
 class CoverageAnalyzeTargetsConfig(CoverageAnalyzeConfig):
     """Configuration for the `coverage analyze targets` command."""
 
 
-def make_report(target_indexes, arcs, lines):  # type: (TargetIndexes, Arcs, Lines) -> t.Dict[str, t.Any]
+def make_report(target_indexes: TargetIndexes, arcs: Arcs, lines: Lines) -> dict[str, t.Any]:
     """Condense target indexes, arcs and lines into a compact report."""
-    set_indexes = {}  # type: TargetSetIndexes
+    set_indexes: TargetSetIndexes = {}
     arc_refs = dict((path, dict((format_arc(arc), get_target_set_index(indexes, set_indexes)) for arc, indexes in data.items())) for path, data in arcs.items())
     line_refs = dict((path, dict((line, get_target_set_index(indexes, set_indexes)) for line, indexes in data.items())) for path, data in lines.items())
 
@@ -47,17 +49,17 @@ def make_report(target_indexes, arcs, lines):  # type: (TargetIndexes, Arcs, Lin
     return report
 
 
-def load_report(report):  # type: (t.Dict[str, t.Any]) -> t.Tuple[t.List[str], Arcs, Lines]
+def load_report(report: dict[str, t.Any]) -> tuple[list[str], Arcs, Lines]:
     """Extract target indexes, arcs and lines from an existing report."""
     try:
-        target_indexes = report['targets']  # type: t.List[str]
-        target_sets = report['target_sets']  # type: t.List[t.List[int]]
-        arc_data = report['arcs']  # type: t.Dict[str, t.Dict[str, int]]
-        line_data = report['lines']  # type: t.Dict[str, t.Dict[int, int]]
+        target_indexes: list[str] = report['targets']
+        target_sets: list[list[int]] = report['target_sets']
+        arc_data: dict[str, dict[str, int]] = report['arcs']
+        line_data: dict[str, dict[int, int]] = report['lines']
     except KeyError as ex:
-        raise ApplicationError('Document is missing key "%s".' % ex.args)
+        raise ApplicationError('Document is missing key "%s".' % ex.args) from None
     except TypeError:
-        raise ApplicationError('Document is type "%s" instead of "dict".' % type(report).__name__)
+        raise ApplicationError('Document is type "%s" instead of "dict".' % type(report).__name__) from None
 
     arcs = dict((path, dict((parse_arc(arc), set(target_sets[index])) for arc, index in data.items())) for path, data in arc_data.items())
     lines = dict((path, dict((int(line), set(target_sets[index])) for line, index in data.items())) for path, data in line_data.items())
@@ -65,20 +67,20 @@ def load_report(report):  # type: (t.Dict[str, t.Any]) -> t.Tuple[t.List[str], A
     return target_indexes, arcs, lines
 
 
-def read_report(path):  # type: (str) -> t.Tuple[t.List[str], Arcs, Lines]
+def read_report(path: str) -> tuple[list[str], Arcs, Lines]:
     """Read a JSON report from disk."""
     try:
         report = read_json_file(path)
     except Exception as ex:
-        raise ApplicationError('File "%s" is not valid JSON: %s' % (path, ex))
+        raise ApplicationError('File "%s" is not valid JSON: %s' % (path, ex)) from None
 
     try:
         return load_report(report)
     except ApplicationError as ex:
-        raise ApplicationError('File "%s" is not an aggregated coverage data file. %s' % (path, ex))
+        raise ApplicationError('File "%s" is not an aggregated coverage data file. %s' % (path, ex)) from None
 
 
-def write_report(args, report, path):  # type: (CoverageAnalyzeTargetsConfig, t.Dict[str, t.Any], str) -> None
+def write_report(args: CoverageAnalyzeTargetsConfig, report: dict[str, t.Any], path: str) -> None:
     """Write a JSON report to disk."""
     if args.explain:
         return
@@ -90,39 +92,39 @@ def write_report(args, report, path):  # type: (CoverageAnalyzeTargetsConfig, t.
     ), verbosity=1)
 
 
-def format_line(value):  # type: (int) -> str
+def format_line(value: int) -> str:
     """Format line as a string."""
     return str(value)  # putting this in a function keeps both pylint and mypy happy
 
 
-def format_arc(value):  # type: (t.Tuple[int, int]) -> str
+def format_arc(value: tuple[int, int]) -> str:
     """Format an arc tuple as a string."""
     return '%d:%d' % value
 
 
-def parse_arc(value):  # type: (str) -> t.Tuple[int, int]
+def parse_arc(value: str) -> tuple[int, int]:
     """Parse an arc string into a tuple."""
     first, last = tuple(map(int, value.split(':')))
     return first, last
 
 
-def get_target_set_index(data, target_set_indexes):  # type: (t.Set[int], TargetSetIndexes) -> int
+def get_target_set_index(data: set[int], target_set_indexes: TargetSetIndexes) -> int:
     """Find or add the target set in the result set and return the target set index."""
     return target_set_indexes.setdefault(frozenset(data), len(target_set_indexes))
 
 
-def get_target_index(name, target_indexes):  # type: (str, TargetIndexes) -> int
+def get_target_index(name: str, target_indexes: TargetIndexes) -> int:
     """Find or add the target in the result set and return the target index."""
     return target_indexes.setdefault(name, len(target_indexes))
 
 
 def expand_indexes(
-        source_data,  # type: IndexedPoints
-        source_index,  # type: t.List[str]
-        format_func,  # type: t.Callable[[TargetKey], str]
-):  # type: (...) -> NamedPoints
+    source_data: IndexedPoints,
+    source_index: list[str],
+    format_func: c.Callable[[TargetKey], TFlexKey],
+) -> dict[str, dict[TFlexKey, set[str]]]:
     """Expand indexes from the source into target names for easier processing of the data (arcs or lines)."""
-    combined_data = {}  # type: t.Dict[str, t.Dict[t.Any, t.Set[str]]]
+    combined_data: dict[str, dict[TFlexKey, set[str]]] = {}
 
     for covered_path, covered_points in source_data.items():
         combined_points = combined_data.setdefault(covered_path, {})
@@ -136,9 +138,9 @@ def expand_indexes(
     return combined_data
 
 
-def generate_indexes(target_indexes, data):  # type: (TargetIndexes, NamedPoints) -> IndexedPoints
+def generate_indexes(target_indexes: TargetIndexes, data: NamedPoints) -> IndexedPoints:
     """Return an indexed version of the given data (arcs or points)."""
-    results = {}  # type: IndexedPoints
+    results: IndexedPoints = {}
 
     for path, points in data.items():
         result_points = results[path] = {}
