@@ -9,8 +9,8 @@ try:
         version,
     )
 
-    SpecifierSet = specifiers.SpecifierSet  # type: t.Optional[t.Type[specifiers.SpecifierSet]]
-    Version = version.Version  # type: t.Optional[t.Type[version.Version]]
+    SpecifierSet: t.Optional[t.Type[specifiers.SpecifierSet]] = specifiers.SpecifierSet
+    Version: t.Optional[t.Type[version.Version]] = version.Version
     PACKAGING_IMPORT_ERROR = None
 except ImportError as ex:
     SpecifierSet = None  # pylint: disable=invalid-name
